@@ -132,11 +132,14 @@ class Molecule():
         molecule : Molecule
         '''
         words = smiles.strip().split()
+        if not words:
+            raise Exception('Invalid SMILES string')
+
         smiles = words[0]
         if len(words) > 1:
             name = words[1]
         else:
-            name = smiles
+            name = None
 
         rdmol = create_mol_from_smiles(smiles)
         mol = Molecule.from_rdmol(rdmol, name)
@@ -248,7 +251,7 @@ class Molecule():
                 Bond.Order.TRIPLE     : Chem.rdchem.BondType.TRIPLE,
             }
             rwmol.AddBond(bond.atom1.id_in_mol, bond.atom2.id_in_mol, d_bond_order[bond.order])
-        Chem.SanitizeMol(rwmol)
+        Chem.SanitizeMol(rwmol) # TODO What is the purpose of this?
         self._rdmol = rwmol.GetMol()
         self._is_rdmol_valid = True
 
